#!/usr/bin/awk
# xconfig.awk - awk script for calculating Xconfig parameters for
# XFree86, given the monitor specs and dot clock frequencies.
# The default values are for the Viewsonic 15.
#
# Example Usage:
#    awk -f xconfig.awk HR=1024 VR=768 DCF=110 < /dev/null (or ^D input)
#
# The variable names and meanings are given below.
#
# Author: Richard Brown rab@tauon.ph.unimelb.edu.au
#

BEGIN{

   # Sensible defaults if not specified:

   HR = 800			# Horizontal Resolution
   VR = 600			# Vertical   Resolution
   DCF = 50.825			# Dot Clock Frequency (MHz)

   HFrontMIN = 0.50		# Horizontal Front Porch Minimum (us)
   HsyncMIN  = 1.20		# Horizontal Sync Pulse Width Minimum (us)
   HBackMIN  = 1.25		# Horizontal Back Porch Minimum (us)
   HBlankMIN = 4.00		# Horizontal Blank Period Minimum (us)
   HSFMAX    = 64.0		# Max Horizontal Scan Frequency (kHz)

   VFrontMIN =   0.0		# Vertical Front Porch Minimum
   VsyncMIN  =  45.0		# Vertical Sync Pulse Width Minimum
   VBackMIN  = 500.0		# Vertical Back Porch Minimum
   VBlankMIN = 600.0		# Vertical Blank Period Minimum
   VSFMAX    = 70.0		# Max Vertical Scan Frequency (Hz)

   verbose = 0

}

END{

   # Horizontal Scan Lines:

   Hfront = HFrontMIN * DCF + HR
   if( Hfront%8 ) Hfront = 8 * (1 + int(Hfront/8) )
   # if( (Hfront - HR) < HFrontMIN * DCF ) Hfront += 8

   Hsync = HsyncMIN * DCF + Hfront
   if( Hsync%8 ) Hsync = 8 * (1 + int(Hsync/8) )

   Hblank = HBlankMIN * DCF
   HFL = HR + Hblank
   if( HFL%8 ) HFL = 8 * (1 + int(HFL/8) )

   # Vertical:

   Vtick = HFL / DCF	# us
   Vfront = VR + VfrontMIN / Vtick
   Vsync = Vfront + VsyncMIN / Vtick
   Vback = VBackMIN / Vtick
   Vblank = VBlankMIN / Vtick
   VFL = Vsync + Vback
   if( VFL < VR + Vblank ) VFL = VR + Vblank

   RR = 1000000 * DCF / (HFL * VFL)
   HSF = 1000 * DCF / HFL

   # Done: generate some output:

   if( verbose ) {
      printf "\n# Horizontal:\n"
      printf "# Front Porch = %.2f us, Sync = %.2f, Back Porch = %.2f, ",
             (Hfront - HR) / DCF, (Hsync - Hfront) / DCF, (HFL - Hsync) / DCF
      printf "Blank = %.2f\n\n", (HFL - HR) / DCF
   }

   printf "# Refresh Rate: %.2f Hz, Horizontal Sync Frequency: %.2f kHz\n",
	  RR, HSF

   printf " \"%dx%d\"\t%.2f\t%d %d %d %d   %d %d %d %d\n", HR, VR, DCF, HR,
	   Hfront, Hsync, HFL, VR, Vfront, Vsync, VFL

   if( HSF > HSFMAX )
      printf("\nWarning: the Horizontal sync frequency may be too high for the monitor!\n")
   if( RR > VSFMAX )
      printf("\nWarning: this refresh rate may be too high for the monitor!\n")
}